require 'pacman.rb'

##
# Trida Figure reprezentuje postavy ve hre.
class Figure
  ##
  # Vytvori novou postavu umestenou na mape v poloze +left+ a +top+,
  # pohybujici se rychlosti +speed+ a vychozim smerem pohybu +r+.
  # * *Args*    :
  #   - +left+ -> poloha na ose X
  #   - +top+ -> poloha na ose Y
  #   - +speed+ -> poloha na ose rychlost pohybu
  #   - +r+ -> vychozi smer pohybu
  def initialize(left, top, speed, r)
    @directions = [:left, :right, :up, :down]
    @left = left
    @top = top
    @future_left = @left
    @future_top = @top
    @speed = speed
    @direction = @directions[r]
  end

  # Smer pohybu
  attr_writer :direction

  def modulo
    if @left > WIDTH
      @left = 0
    elsif @left < 0
      @left = WIDTH
    end
  end

  ##
  # Overi, jestli se postava puze dale pohybovat soucasnym smerem
  # v oblasti +area+
  # * *Args*    :
  #   - +area+ -> mapa hry
  # * *Returns* :
  #   - true -> postava se muze dal pohybovat stejnym smrem
  #   - false -> postava musi zmeni smer pohybu
  def can_move(area)
    future_move
    area.colidate(@future_left + CROFTWIDTH / 2, @future_top + CROFTWIDTH / 2)
  end

  ##
  # Posune hrace soucasnym smerem
  def move
    real_move
    modulo
    future_make
  end

  def real_move
    if @direction == :left
      @left -= speed
    elsif @direction == :right
      @left += speed
    elsif @direction == :up
      @top -= speed
    else
      @top += speed
    end
  end

  def future_make
    @future_left = @left
    @future_top = @top
  end

  def future_move
    future_make
    if @direction == :left
      @future_left -= speed
    elsif @direction == :right
      @future_left += speed
    elsif @direction == :up
      @future_top -= speed
    else
      @future_top += speed
    end
  end

  private :future_move, :future_make, :real_move, :modulo

  # Pozice na ose X
  attr_reader :left

  # Pozice na ose Y
  attr_reader :top

  # Rychlost pohybu
  attr_reader :speed

  attr_reader :direction
end

##
# Trida Ghost reprezentuje pocitacem ovladane duchy.
class Ghost < Figure
  ##
  # Vytvori noveho ducha umesteneho na mape v poloze +left+ a +top+,
  # pohybujiciho se rychlosti +speed+ a vychozim smerem pohybu +r+.
  # * *Args*    :
  #   - +left+ -> poloha na ose X
  #   - +top+ -> poloha na ose Y
  #   - +speed+ -> poloha na ose rychlost pohybu
  #   - +r+ -> vychozi smer pohybu
  def initialize(left, top, speed, r)
    super(left, top, speed, r)
    @am_i_blue = false
  end

  # reprezentuje stav ducha
  attr_reader :am_i_blue

  # duch ma moznost zmenit rychlost pohybu
  attr_writer :am_i_blue, :speed

  ##
  # Pokud se duch srazi s hracem +p+ tak ho bud sni (hra konci),
  # nebo je snezen a vrati se na vychozi umisteni.
  # * *Args*    :
  #   - +p+ -> postava pacmana
  # * *Returns* :
  #   - GAME_OVER -> pacman byl snezen a hra konci
  #   - I GO HOME -> duch byl snezen a vraci se
  def eat_pacman(p)
    c = CROFTWIDTH
    return unless (@left - p.left).abs < c - 18 && (@top - p.top).abs < c - 18
    if !am_i_blue
      'GAME_OVER'
    else
      'I GO HOME'
    end
  end

  ##
  # Vyzkousi kam je mozne se pohybovat a nastavi novy smer pohybu
  # * *Args*    :
  #   - +area+ -> mapa hry
  def try_set_direction(area)
    original_direction = @direction
    possible_moves = []
    @directions.each do |dir|
      @direction = dir
      possible_moves.push(dir) if can_move(area)
    end
    select_direction(original_direction, possible_moves)
  end

  def select_direction(original_direction, possible_moves)
    if possible_moves.length == 1 || possible_moves.empty?
      @direction = possible_moves[0]
    else
      r = Random.rand(possible_moves.length)
      while oposit?(original_direction, possible_moves[r])
        r = Random.rand(possible_moves.length)
      end
      @direction = possible_moves[r]
    end
  end

  def oposit?(org, dir)
    return true if compare_l_r(org, dir) || compare_t_d(org, dir)
    false
  end

  def compare_t_d(org, dir)
    td1 = org == :up && dir == :down
    td2 = org == :down && dir == :up
    return true if td1 || td2
    false
  end

  def compare_l_r(org, dir)
    lr1 = org == :left && dir == :right
    lr2 = org == :right && dir == :left
    return true if lr1 || lr2
    false
  end

  private :compare_l_r, :compare_t_d, :oposit?, :select_direction
end

##
# Trida Player predstavuje postavu ovladanou hracem (PACMANA)
class Player < Figure
  ##
  # Vytvori noveho hrace umesteneho na mape v poloze +left+ a +top+,
  # pohybujiciho se rychlosti +speed+ a vychozim smerem pohybu +r+.
  # * *Args*    :
  #   - +left+ -> poloha na ose X
  #   - +top+ -> poloha na ose Y
  #   - +speed+ -> poloha na ose rychlost pohybu
  #   - +r+ -> vychozi smer pohybu
  def initialize(left, top, speed, r)
    super(left, top, speed, r)
    @score = 0
  end

  ##
  # Zkontroluje, jestli hrac sebral bonus v mape +area+.
  # Pokud ano, zvysi skore.
  # * *Args*    :
  #   - +area+ -> mapa hry
  def collect_bonus(area)
    score = area.bonus(@left + CROFTWIDTH / 2, @top + CROFTWIDTH / 2)
    @score += score
    score
  end

  ##
  # Pricte skore za znezeni ducha.
  def you_eat_them
    @score += 100
  end

  # reprezentuje hodnotu skore, ktereho hrac dosahl
  attr_reader :score
end
