require 'figure.rb'
require 'croft.rb'
require 'area.rb'

# Velikost jednoho policka na mape
CROFTWIDTH = 30

# Pozice pro zacatek vykreslovani na ose Y
Y_START = 0

# Sirka okna hry
WIDTH = 23 * CROFTWIDTH

# Vyska okna hry
HEIGHT = 24 * CROFTWIDTH + Y_START

##
# Trida Pacman je hlavni trida ktera reprezentuje celou hru.
class Pacman
  ##
  # Vytvori novou instanci hry
  def initialize
    @pc_speed = 10
    @gh_sp = 15
    @time_mark = 0
    @area = Area.new
    @ghosts = []
    make_ghosts
    @player = Player.new(@area.pc_pos[:x], @area.pc_pos[:y], @pc_speed, 1)
  end

  def make_ghosts
    4.times do
      @ghosts.push Ghost.new(@area.gh_pos[:x], @area.gh_pos[:y], @gh_sp, 2)
    end
  end

  ##
  # Zmeni smer pohybu hrace na +direction+
  # * *Args*    :
  #   - +direction+ -> novy smer pohybu hrace
  def move(direction)
    @player.direction = direction
  end

  ##
  # Aktualizuje stav hry
  # * *Returns* :
  #   - GAME_OVER -> Konec hry
  def update
    try_time
    @player.move if @player.can_move(@area)
    if @player.collect_bonus(@area) == 10
      make_ghosts_blue
      @time_mark = Time.now
    end
    update_ghosts
  end

  def update_ghosts
    @ghosts.each do |ghost|
      ghost.try_set_direction(@area)
      ghost.move
      if decide(ghost.eat_pacman(@player), @ghosts.index(ghost)) == true
        return 'GAME_OVER'
      end
    end
  end

  def try_time
    return unless @time_mark != 0 && Time.now - @time_mark >= 5
    @ghosts.each do |ghost|
      ghost.am_i_blue = false
      ghost.speed = @gh_sp
    end
    @time_mark = 0
  end

  def decide(value, ghost)
    return true if value == 'GAME_OVER'
    return unless value == 'I GO HOME'
    @ghosts[ghost] = Ghost.new(@area.gh_pos[:x], @area.gh_pos[:y], @gh_sp, 2)
    @player.you_eat_them
    false
  end

  def make_ghosts_blue
    @ghosts.each do |ghost|
      ghost.am_i_blue = true
      ghost.speed = 5
    end
  end

  private :make_ghosts, :make_ghosts_blue, :update_ghosts, :decide, :try_time

  # pole duchu ve hre
  attr_reader :ghosts

  # mapa hry
  attr_reader :area

  # postava ovladana hracem
  attr_reader :player
end
