require 'pacman_semestral/version'

require 'green_shoes'
require 'figure.rb'

Shoes.app height: HEIGHT, width: WIDTH do
  @game = Pacman.new
  @ghs = @game.ghosts
  @plr = @game.player
  background black
  keypress do |k|
    case k
    when 'Left' then @game.move(:left)
    when 'Right' then @game.move(:right)
    when 'Down' then @game.move(:down)
    when 'Up' then @game.move(:up)
    end
  end
  @c = CROFTWIDTH
  cm = @c - 10
  @player = oval radius: @c / 2, center: false, fill: yellow
  @player.move(@plr.left, @plr.top)
  @red = rect @ghs[0].left + 5, @ghs[0].top + 5, cm, cm, fill: red
  @pink = rect @ghs[1].left + 5, @ghs[1].top + 5, cm, cm, fill: deeppink
  @green = rect @ghs[2].left + 5, @ghs[2].top + 5, cm, cm, fill: green
  @orange = rect @ghs[3].left + 5, @ghs[3].top + 5, cm, cm, fill: orange
  @bonuses = []

  ##
  # vykresli mapu
  def area_build
    @game.area.crofts.each do |line_crofts|
      line_crofts.each do |croft|
        rect croft.left, croft.top, @c, @c, fill: blue if croft.type == '#'
      end
    end
    draw_bonuses
  end

  ##
  # vykresli vsechny postavy
  def draw_figures
    move(@player, @plr.left, @plr.top)
    move(@red, @ghs[0].left + 5, @ghs[0].top + 5)
    move(@pink, @ghs[1].left + 5, @ghs[1].top + 5)
    move(@green, @ghs[2].left + 5, @ghs[2].top + 5)
    move(@orange, @ghs[3].left + 5, @ghs[3].top + 5)
    draw_colors
  end

  ##
  # Posune obraz postavy
  # * *Args*    :
  #   - +itm+ -> postava
  #   - +left+ -> poloha na ose X
  #   - +top+ -> poloha na ose Y
  def move(itm, left, top)
    itm.left = left
    itm.top = top
  end

  ##
  # vykresli barvy pro vsechny duchy
  def draw_colors
    draw(@red, 0, red)
    draw(@pink, 1, deeppink)
    draw(@green, 2, green)
    draw(@orange, 3, orange)
  end

  ##
  # vykresli barvu ducha
  # * *Args*    :
  #   - +ghost+ -> duch
  #   - +ind+ -> index ducha
  #   - +color+ -> barva ducha
  def draw(ghost, ind, color)
    ghost.style(fill: blue) if @ghs[ind].am_i_blue
    ghost.style(fill: color) unless @ghs[ind].am_i_blue
  end

  ##
  # vykresli bonusy
  def draw_bonuses
    @game.area.crofts.each do |line_crofts|
      line_crofts.each do |cr|
        t = cr.top + @c / 3
        push_small_bonus(cr, t)
        push_big_bonus(cr, t)
      end
    end
  end

  ##
  # prida do pole bonusu maly bonus
  def push_small_bonus(cr, t)
    return unless cr.type == '*'
    o = oval left: cr.left + @c / 3, top: t, radius: @c / 6, fill: white
    @bonuses.push(o)
    cr.index = @bonuses.length - 1
  end

  ##
  # prida do pole bonusu velky bonus
  def push_big_bonus(cr, t)
    return unless cr.type == '@'
    o = oval left: cr.left + @c / 3.8, top: t, radius: @c / 4, fill: red
    @bonuses.push(o)
    cr.index = @bonuses.length - 1
  end

  ##
  # prekresli bonusy
  def update_bonuses
    @game.area.crofts.each do |line_crofts|
      line_crofts.each do |croft|
        @bonuses[croft.index].clear if croft.visited
      end
    end
  end

  area_build
  animate 8 do
    if @game.update == 'GAME_OVER'
      alert('GAME OVER   SCORE=' + @plr.score.to_s)
      break
    end
    update_bonuses
    draw_figures
  end
end
